import fs from 'fs';
import normalizeNewline from 'normalize-newline';
import path from 'path';
// TODO: add exports
const FIELDS = [
    'main',
    'bin',
    'types',
    'typings',
    'module',
    'es2015',
    'browser',
    'exports', // https://nodejs.org/api/packages.html#subpath-exports
];
// Check fields for file existence
function doesNotExist(dir, file) {
    return !fs.existsSync(path.join(dir, file));
}
function checkField(pkg, dir, field) {
    const errors = [];
    if (pkg[field]) {
        if (pkg[field] instanceof Object) {
            Object.keys(pkg[field]).forEach((key) => {
                if (doesNotExist(dir, pkg[field][key])) {
                    errors.push(`${field}.${key}`);
                }
            });
        }
        else {
            if (doesNotExist(dir, pkg[field])) {
                errors.push(field);
            }
        }
    }
    return errors;
}
function checkFields(pkg, dir, otherFields) {
    const fields = [...FIELDS, ...otherFields];
    // Check fields and add errors to the errors array
    return fields.flatMap((field) => checkField(pkg, dir, field));
}
// Check scripts line endings
function normalize(dir, file) {
    const filename = path.join(dir, file);
    const data = fs.readFileSync(filename, 'utf-8');
    const normalizedData = normalizeNewline(data);
    fs.writeFileSync(filename, normalizedData);
}
function normalizeField(pkg, dir, field) {
    if (pkg[field]) {
        if (pkg[field] instanceof Object) {
            Object.keys(pkg[field]).forEach((key) => normalize(dir, pkg[field][key]));
        }
        else {
            normalize(dir, pkg[field]);
        }
    }
}
function normalizeScripts(pkg, dir, files) {
    normalizeField(pkg, dir, 'bin');
    files.forEach((file) => normalize(dir, file));
}
// Main function
export function pkgOk(dir, { fields = [], bin = [] } = {}) {
    const pkgPath = path.join(dir, 'package.json');
    const pkg = JSON.parse(fs.readFileSync(pkgPath).toString());
    // Check files exist in package.json fields and additional fields
    const errors = checkFields(pkg, dir, fields);
    if (errors.length) {
        const message = errors.map((error) => `${error} path doesn't exist in package.json`).join('\n');
        throw new Error(message);
    }
    // Normalize line endings for bin scripts and additional scripts
    normalizeScripts(pkg, dir, bin);
}
//# sourceMappingURL=index.js.map